@@ -1,13 +1,15 @@
-#!/usr/bin/env python
+#!/usr/local/bin/python2.5
 #
-#  hot-backup.py: perform a "hot" backup of a Berkeley DB repository.
-#                 (and clean old logfiles after backup completes.)
+#  hot-backup.py: perform a "hot" backup of a Subversion repository
+#                 and clean any old Berkeley DB logfiles after the
+#                 backup completes, if the repository backend is
+#                 Berkeley DB.
 #
 #  Subversion is a tool for revision control. 
 #  See http://subversion.tigris.org for more information.
 #    
 # ====================================================================
-# Copyright (c) 2000-2006 CollabNet.  All rights reserved.
+# Copyright (c) 2000-2007 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -20,21 +22,38 @@
 # history and logs, available at http://subversion.tigris.org/.
 # ====================================================================
 
+# $HeadURL: http://svn.collab.net/repos/svn/branches/1.5.x/tools/backup/hot-backup.py.in $
+# $LastChangedDate: 2007-10-26 00:11:26 +0000 (Fri, 26 Oct 2007) $
+# $LastChangedBy: kfogel $
+# $LastChangedRevision: 27407 $
+
 ######################################################################
 
-import sys, os, getopt, shutil, string, re
+import sys, os, getopt, stat, string, re, time, shutil
+
+# Try to import the subprocess mode.  It works better then os.popen3
+# and os.spawnl on Windows when spaces appear in any of the svnadmin,
+# svnlook or repository paths.  os.popen3 and os.spawnl are still used
+# to support Python 2.3 and older which do not provide the subprocess
+# module.  have_subprocess is set to 1 or 0 to support older Python
+# versions that do not have True and False.
+try:
+  import subprocess
+  have_subprocess = 1
+except ImportError:
+  have_subprocess = 0
 
 ######################################################################
 # Global Settings
 
 # Path to svnlook utility
-svnlook = "/usr/local/bin/svnlook"
+svnlook = r"/usr/local/bin/svnlook"
 
 # Path to svnadmin utility
-svnadmin = "/usr/local/bin/svnadmin"
+svnadmin = r"/usr/local/bin/svnadmin"
 
-# Number of backups to keep around (0 for "keep them all")
-num_backups = 64
+# Default number of backups to keep around (0 for "keep them all")
+num_backups = int(os.environ.get("SVN_HOTBACKUP_BACKUPS_NUMBER", 64))
 
 # Archive types/extensions
 archive_map = {
@@ -43,6 +62,39 @@ archive_map = {
   'zip' : ".zip"
   }
 
+# Chmod recursively on a whole subtree
+def chmod_tree(path, mode, mask):
+  def visit(arg, dirname, names):
+    mode, mask = arg
+    for name in names:
+      fullname = os.path.join(dirname, name)
+      if not os.path.islink(fullname):
+        new_mode = (os.stat(fullname)[stat.ST_MODE] & ~mask) | mode
+        os.chmod(fullname, new_mode)
+  os.path.walk(path, visit, (mode, mask))
+
+# For clearing away read-only directories
+def safe_rmtree(dirname, retry=0):
+  "Remove the tree at DIRNAME, making it writable first"
+  def rmtree(dirname):
+    chmod_tree(dirname, 0666, 0666)
+    shutil.rmtree(dirname)
+
+  if not os.path.exists(dirname):
+    return
+
+  if retry:
+    for delay in (0.5, 1, 2, 4):
+      try:
+        rmtree(dirname)
+        break
+      except:
+        time.sleep(delay)
+    else:
+      rmtree(dirname)
+  else:
+    rmtree(dirname)
+
 ######################################################################
 # Command line arguments
 
@@ -59,13 +111,16 @@ Options:
                        bz2 : Creates a bzip2 compressed tar file.
                        gz  : Creates a gzip compressed tar file.
                        zip : Creates a compressed zip file.
+  --num-backups=N    Number of prior backups to keep around (0 to keep all).
   --help      -h     Print this help message and exit.
 
 """ % (scriptname,))
 
 
 try:
-  opts, args = getopt.gnu_getopt(sys.argv[1:], "h?", ["archive-type=", "help"])
+  opts, args = getopt.gnu_getopt(sys.argv[1:], "h?", ["archive-type=",
+                                                      "num-backups=",
+                                                      "help"])
 except getopt.GetoptError, e:
   print >> sys.stderr, "ERROR: %s\n" % e
   usage(sys.stderr)
@@ -76,6 +131,8 @@ archive_type = None
 for o, a in opts:
   if o == "--archive-type":
     archive_type = a
+  elif o == "--num-backups":
+    num_backups = int(a)
   elif o in ("-h", "--help", "-?"):
     usage()
     sys.exit()
@@ -135,6 +192,31 @@ def comparator(a, b):
     else:
       return 1
 
+def get_youngest_revision():
+  """Examine the repository REPO_DIR using the svnlook binary
+  specified by SVNLOOK, and return the youngest revision."""
+
+  if have_subprocess:
+    p = subprocess.Popen([svnlook, 'youngest', repo_dir],
+                         stdin=subprocess.PIPE,
+                         stdout=subprocess.PIPE,
+                         stderr=subprocess.PIPE)
+    infile, outfile, errfile = p.stdin, p.stdout, p.stderr
+  else:
+    infile, outfile, errfile = os.popen3(svnlook + " youngest " + repo_dir)
+
+  stdout_lines = outfile.readlines()
+  stderr_lines = errfile.readlines()
+  outfile.close()
+  infile.close()
+  errfile.close()
+
+  if stderr_lines:
+    raise Exception("Unable to find the youngest revision for repository '%s'"
+                    ": %s" % (repo_dir, string.rstrip(stderr_lines[0])))
+
+  return string.strip(stdout_lines[0])
+
 ######################################################################
 # Main
 
@@ -143,14 +225,12 @@ print "Beginning hot backup of '"+ repo_dir + "'."
 
 ### Step 1: get the youngest revision.
 
-infile, outfile, errfile = os.popen3(svnlook + " youngest " + repo_dir)
-stdout_lines = outfile.readlines()
-stderr_lines = errfile.readlines()
-outfile.close()
-infile.close()
-errfile.close()
+try:
+  youngest = get_youngest_revision()
+except Exception, e:
+  print >> sys.stderr, str(e)
+  sys.exit(1)
 
-youngest = string.strip(stdout_lines[0])
 print "Youngest revision is", youngest
 
 
@@ -182,8 +262,12 @@ if young_list:
 ###         copied last.
 
 print "Backing up repository to '" + backup_subdir + "'..."
-err_code = os.spawnl(os.P_WAIT, svnadmin, "svnadmin", "hotcopy", repo_dir, 
-                     backup_subdir, "--clean-logs")
+if have_subprocess:
+  err_code = subprocess.call([svnadmin, "hotcopy", repo_dir, 
+                              backup_subdir, "--clean-logs"])
+else:
+  err_code = os.spawnl(os.P_WAIT, svnadmin, "svnadmin", "hotcopy", repo_dir, 
+                       backup_subdir, "--clean-logs")
 if err_code != 0:
   print >> sys.stderr, "Unable to backup the repository."
   sys.exit(err_code)
@@ -242,7 +326,7 @@ if archive_type:
     sys.exit(err_code)
   else:
     print "Archive created, removing backup '" + backup_subdir + "'..."
-    shutil.rmtree(backup_subdir)
+    safe_rmtree(backup_subdir, 1)
 
 ### Step 5: finally, remove all repository backups other than the last
 ###         NUM_BACKUPS.
@@ -257,6 +341,6 @@ if num_backups > 0:
     old_backup_item = os.path.join(backup_dir, item)
     print "Removing old backup: " + old_backup_item
     if os.path.isdir(old_backup_item):
-      shutil.rmtree(old_backup_item)
+      safe_rmtree(old_backup_item, 1)
     else:
       os.remove(old_backup_item)
