@@ -9,10 +9,10 @@
 #
 # This script requires Subversion 1.2.0 or later.
 #
-# $HeadURL: http://svn.collab.net/repos/svn/branches/1.4.x/tools/hook-scripts/commit-email.pl.in $
-# $LastChangedDate: 2006-04-19 23:08:55 +0000 (Wed, 19 Apr 2006) $
-# $LastChangedBy: maxb $
-# $LastChangedRevision: 19424 $
+# $HeadURL: http://svn.collab.net/repos/svn/branches/1.5.x/tools/hook-scripts/commit-email.pl.in $
+# $LastChangedDate: 2008-04-01 15:12:17 +0000 (Tue, 01 Apr 2008) $
+# $LastChangedBy: glasser $
+# $LastChangedRevision: 30155 $
 #
 # ====================================================================
 # Copyright (c) 2000-2006 CollabNet.  All rights reserved.
@@ -38,6 +38,7 @@ BEGIN {
 
 use strict;
 use Carp;
+use POSIX qw(strftime);
 my ($sendmail, $smtp_server);
 
 ######################################################################
@@ -47,6 +48,7 @@ my ($sendmail, $smtp_server);
 # You should define exactly one of these two configuration variables,
 # leaving the other commented out, to select which method of sending
 # email should be used.
+# Using --stdout on the command line overrides both.
 #$sendmail = "/usr/sbin/sendmail";
 $smtp_server = "127.0.0.1";
 
@@ -123,6 +125,7 @@ my $author;
 my $propname;
 
 my $mode = 'commit';
+my $date;
 my $diff_file;
 
 # Use the reference to the first project to populate.
@@ -139,7 +142,9 @@ my %opt_to_hash_key = ('--from' => 'from_address',
                        '-m'     => '',
                        '-r'     => 'reply_to',
                        '-s'     => 'subject_prefix',
-                       '--diff' => '');
+                       '--summary' => '',
+                       '--diff' => '',
+                       '--stdout' => '');
 
 while (@ARGV)
   {
@@ -153,7 +158,7 @@ while (@ARGV)
           }
 
         my $value;
-        if ($arg ne '--revprop-change')
+        if ($arg ne '--revprop-change' and $arg ne '--stdout' and $arg ne '--summary')
           {
             unless (@ARGV)
               {
@@ -201,6 +206,14 @@ while (@ARGV)
               {
                 $current_project->{show_diff} = parse_boolean($value);
               }
+            elsif ($arg eq '--stdout')
+              {
+                $current_project->{stdout} = 1;
+              }
+            elsif ($arg eq '--summary')
+              {
+                $current_project->{summary} = 1;
+              }
             else
               {
                 die "$0: internal error:"
@@ -343,6 +356,7 @@ foreach my $line (@svnlooklines)
 # Declare variables which carry information out of the inner scope of
 # the conditional blocks below.
 my $subject_base;
+my $subject_logbase;
 my @body;
 # $author - declared above for use as a command line parameter in
 #   revprop-change mode.  In commit mode, gets filled in below.
@@ -355,7 +369,7 @@ if ($mode eq 'commit')
     # Get the author, date, and log from svnlook.
     my @infolines = &read_from_process($svnlook, 'info', $repos, '-r', $rev);
     $author = shift @infolines;
-    my $date = shift @infolines;
+    $date = shift @infolines;
     shift @infolines;
     my @log = map { "$_\n" } @infolines;
 
@@ -420,6 +434,9 @@ if ($mode eq 'commit')
       {
         $subject_base = "r$rev - $dirlist";
       }
+    my $summary = @log ? $log[0] : '';
+    chomp($summary);
+    $subject_logbase = "r$rev - $summary";
 
     # Put together the body of the log message.
     push(@body, "Author: $author\n");
@@ -513,9 +530,11 @@ foreach my $project (@project_settings_list)
     my $log_file        = $project->{log_file};
     my $reply_to        = $project->{reply_to};
     my $subject_prefix  = $project->{subject_prefix};
-    my $subject         = $subject_base;
+    my $summary         = $project->{summary};
     my $diff_wanted     = ($project->{show_diff} and $mode eq 'commit');
+    my $stdout          = $project->{stdout};
 
+    my $subject         = $summary ? $subject_logbase : $subject_base;
     if ($subject_prefix =~ /\w/)
       {
         $subject = "$subject_prefix $subject";
@@ -530,13 +549,21 @@ foreach my $project (@project_settings_list)
       {
         $mail_from = "$mail_from\@$hostname";
       }
-    elsif (defined $smtp_server)
+    elsif (defined $smtp_server and ! $stdout)
       {
         die "$0: use of either `-h' or `--from' is mandatory when ",
             "sending email using direct SMTP.\n";
       }
 
     my @head;
+    my $formatted_date;
+    if (defined $stdout)
+      {
+        $formatted_date = strftime('%a %b %e %X %Y', localtime());
+        push(@head, "From $mail_from $formatted_date\n");
+      }
+    $formatted_date = strftime('%a, %e %b %Y %X %z', localtime());
+    push(@head, "Date: $formatted_date\n");
     push(@head, "To: $to\n");
     push(@head, "From: $mail_from\n");
     push(@head, "Subject: $subject\n");
@@ -583,7 +610,12 @@ foreach my $project (@project_settings_list)
         @difflines = map { /[\r\n]+$/ ? $_ : "$_\n" } @difflines;
       }
 
-    if (defined $sendmail and @email_addresses)
+    if ($stdout)
+      {
+        print @head, @body;
+        print @difflines if $diff_wanted;
+      }
+    elsif (defined $sendmail and @email_addresses)
       {
         # Open a pipe to sendmail.
         my $command = "$sendmail -f'$mail_from' $userlist";
@@ -601,7 +633,8 @@ foreach my $project (@project_settings_list)
       }
     elsif (defined $smtp_server and @email_addresses)
       {
-        my $smtp = Net::SMTP->new($smtp_server);
+        my $smtp = Net::SMTP->new($smtp_server)
+          or die "$0: error opening SMTP session to `$smtp_server': $!\n";
         handle_smtp_error($smtp, $smtp->mail($mail_from));
         handle_smtp_error($smtp, $smtp->recipient(@email_addresses));
         handle_smtp_error($smtp, $smtp->data());
@@ -657,8 +690,10 @@ sub usage
       "  -m regex              Regular expression to match committed path\n",
       "  -r email_address      Email address for 'Reply-To:'\n",
       "  -s subject_prefix     Subject line prefix\n",
+      "  --summary             Use first line of commit log in subject\n",
       "  --diff y|n            Include diff in message (default: y)\n",
       "                        (applies to commit mode only)\n",
+      "  --stdout              Spit the message in mbox format to stdout.\n",
       "\n",
       "This script supports a single repository with multiple projects,\n",
       "where each project receives email only for actions that affect that\n",
@@ -667,10 +702,10 @@ sub usage
       "contains modifications to a path that matches the regular\n",
       "expression, then the action applies to the project.\n",
       "\n",
-      "Any of the following -h, -l, -r, -s and --diff command line options\n",
-      "and following email addresses are associated with this project.  The\n",
-      "next -m resets the -h, -l, -r, -s and --diff command line options\n",
-      "and the list of email addresses.\n",
+      "Any of the following -h, -l, -r, -s, --summary, and --diff command\n",
+      "line options and following email addresses are associated with\n",
+      "this project.  The next -m resets the -h, -l, -r, -s, --summary,\n",
+      "and --diff command line options and the list of email addresses.\n",
       "\n",
       "To support a single project conveniently, the script initializes\n",
       "itself with an implicit -m . rule that matches any modifications\n",
@@ -698,7 +733,8 @@ sub new_project
           match_regex     => '.',
           reply_to        => '',
           subject_prefix  => '',
-          show_diff       => 1};
+          show_diff       => 1,
+          stdout          => 0};
 }
 
 sub parse_boolean
@@ -717,17 +753,37 @@ sub safe_read_from_pipe
       croak "$0: safe_read_from_pipe passed no arguments.\n";
     }
 
-  my $pid = open(SAFE_READ, '-|');
-  unless (defined $pid)
+  my $openfork_available = $^O ne "MSWin32"; 
+  if ($openfork_available) # We can fork on this system.
     {
-      die "$0: cannot fork: $!\n";
+      my $pid = open(SAFE_READ, '-|');
+      unless (defined $pid)
+        {
+          die "$0: cannot fork: $!\n";
+        }
+      unless ($pid)
+        {
+          open(STDERR, ">&STDOUT")
+            or die "$0: cannot dup STDOUT: $!\n";
+          exec(@_)
+            or die "$0: cannot exec `@_': $!\n";
+        }
     }
-  unless ($pid)
+  else  # Running on Windows.  No fork. 
     {
-      open(STDERR, ">&STDOUT")
-        or die "$0: cannot dup STDOUT: $!\n";
-      exec(@_)
-        or die "$0: cannot exec `@_': $!\n";
+      my @commandline = ();
+      my $arg;
+      
+      while ($arg = shift)
+        {
+          $arg =~ s/\"/\\\"/g;
+          if ($arg eq "" or $arg =~ /\s/) { $arg = "\"$arg\""; }
+          push(@commandline, $arg);
+        }
+        
+      # Now do the pipe.
+      open(SAFE_READ, "@commandline |")
+        or die "$0: cannot pipe to command: $!\n";
     }
   my @output;
   while (<SAFE_READ>)
